﻿var OpenSearchFinder = function(){
	this._elements = null;
	this._elementsProfile = [];
	this._cursor = -1;
	this._inProgress = false;
	
	this._timer = null;
}
OpenSearchFinder.PATTERN_QUERY_TERMS = /(\{.+\})/gi
OpenSearchFinder.prototype = {
	find:function(){
		this._elements = this._findHTMLDeclaration();
		this.getProfile();
	},
	getProfile:function(){
		var _this = this;
		if(this._cursor >= this._elements.length - 1){
			if(_this._elementsProfile.length == 0)
				return;
			localStorage.setItem("data",JSON.stringify(_this._elementsProfile));
			chrome.extension.sendRequest({
				command:"FOUND_OPEN_SEARCH"
			},function(){
			});
			return;
		}
		this._cursor++;
		
		var url = this._elements[this._cursor];
		var xhr = new XMLHttpRequest();
		xhr.onreadystatechange = function(){
			if(xhr.readyState == 4){
				if(xhr.status == 200){
					if(_this._timer)
						clearTimeout(_this._timer);
					_this._timer = null;
					var doc = xhr.responseXML,encoding = doc.documentElement.querySelector("InputEncoding");
					//filter encoding
					var nameNode = doc.documentElement.querySelector("ShortName"),urlNode = doc.documentElement.querySelector("Url"),url = "";
					OpenSearchFinder.PATTERN_QUERY_TERMS.lastIndex = 0;
					if(urlNode && urlNode.getAttribute("template").length > 0){
						url = urlNode.getAttribute("template");
						if(OpenSearchFinder.PATTERN_QUERY_TERMS.test(urlNode.getAttribute("template"))){
							//done
						}else{
							var params = urlNode.querySelectorAll("Param");
							if(params.length > 0){
								if(url.indexOf("?") == -1)
									url += "?";
								for(var i=0,len=params.length;i<len;i++){
									url += "&"+params[i].getAttribute("name")+"="+params[i].getAttribute("value");
								}
							}
						}
					}
					_this._elementsProfile.push({
						name : nameNode ? nameNode.textContent : "Untitled",
						url : url
					});
					_this._inProgress = false;
					_this.getProfile();
				}
			}
		}
		xhr.open("GET",url,true);
		xhr.send(null);
		this._inProgress = true;
		this._timer = setTimeout(function(){
			if(_this._inProgress){
				xhr.abort();
			}
			this._timer = null;
		},30000);
	},
	_findHTMLDeclaration:function(){
		var list = document.documentElement.querySelectorAll("[type='application/opensearchdescription+xml'][rel='search']");
		var result = [];
		for(var i=0,len=list.length;i<len;i++){
			result.push(list[i].href);
		}
		return result;
	}
}

/**/
var FeedFinder = function() {
	this._mFeeds = null;
};
FeedFinder.KEY_FEED = "_cd_pref_feeds";
FeedFinder.KEY_FEED_FIND_TIME = "_cd_pref_feed_find_at";

FeedFinder.KEY_LAST_FOUND = "_cd_pref_last_found";
FeedFinder.KEY_FEED_CHANGE_COUNT = "_cd_pref_change_count";

//update period
FeedFinder.FIND_PERIOD = 6 * 24 * 60 * 60 * 1000;
FeedFinder.FIND_PERIOD = 1;

FeedFinder.listFeeds = function() {
	var feeds = document.documentElement.querySelectorAll("link[type='"+ 
			FeedFinder.MIME_RSS +"']"), 
	FOR_EACH = Array.prototype.forEach, 
	list = [];
	FOR_EACH.call(feeds, function(item, idx){
		var data = {};
		data["title"] = item.title ? item.title : "untitled";
		data["url"] = item.href
		list.push(data);
	});
	
	if (list.length == 0) {
		//TODO try to figure out SNS feed
		var sns = SNSFeedManager.getFeed(
			[null, null, null, location.href, null],
			null
		);
		if (sns != null) {
//			console.log("Hit a SNS Feed");
			var data = {};
			data["title"] = document.title;
			data["url"] = location.href;
			list.push(data);
		}
	}
	
	return list;
}
FeedFinder.isCheatingFeed = function() {
	//TODO frequently change means the site use a dynamic feed address
	// we use the last found
	var count = localStorage.getItem(FeedFinder.KEY_FEED_CHANGE_COUNT);
//	console.log(">>>>>>>cheating count is:", count);
	if (!count)
		return false;
	return parseInt(count) >= 2;
}
FeedFinder.increseCheatingCount = function() {
	var count = localStorage.getItem(FeedFinder.KEY_FEED_CHANGE_COUNT);
	if (!count)
		count = 0;
	localStorage.setItem(FeedFinder.KEY_FEED_CHANGE_COUNT, ++count);
}
FeedFinder.getLastFound = function() {
	var f = localStorage.getItem(FeedFinder.KEY_LAST_FOUND);
	if (!f)
		return null;
	else
		return f;
}
FeedFinder.testCheating = function(newFeed) {
	var feed = FeedFinder.getLastFound();
	if (!feed) {
		localStorage.setItem(FeedFinder.KEY_LAST_FOUND, newFeed);
		return;
	}
	if (newFeed != feed) {
		FeedFinder.increseCheatingCount();
		localStorage.setItem(FeedFinder.KEY_LAST_FOUND, newFeed);
	}
}
FeedFinder.MIME_RSS = "application/rss+xml";
FeedFinder.prototype = {
	needFind : function() {
		var find = localStorage.getItem(FeedFinder.KEY_FEED_FIND_TIME), 
		need = false, 
		now = new Date().getTime();
		if (!find) {
			need = true;
		} else {
			var time = parseInt(find);
			if (now - time > FeedFinder.FIND_PERIOD)
				need = true;
		}
		
		localStorage.setItem(FeedFinder.KEY_FEED_FIND_TIME, now);
		return need;
	},
	find : function() {
		//TODO find anyway
		var newFeed = JSON.stringify(FeedFinder.listFeeds()), result = null;
		if (this.needFind()) {
//			console.log(">>>>>need find");
			if (FeedFinder.isCheatingFeed()) {
				var lastFound = FeedFinder.getLastFound();
//				console.log(">>>>and last found is:", lastFound);
				if (lastFound)
					return lastFound;//reject finding process, do not update feed anymore
				else
					result = newFeed;
			} else {
				result = newFeed;	
			}
		} else {
//			console.log("no need find now");
			result = "";
		}
//		console.log(">>>>BEFORE RETURN FEED:", result);
		FeedFinder.testCheating(newFeed);
//		console.log(">>>FOUND FEED:", result);
		return result;
//		return true || this.needFind() ? JSON.stringify(FeedFinder.listFeeds()) : "";
	}
}

chrome.extension.onConnect.addListener(function(port){
	port.onMessage.addListener(function(data){
		if(data.command == "FIND_OPEN_SEARCH"){
			new OpenSearchFinder().find();
		}else if(data.command == "RETRIEVE_OPEN_SEARCH"){
			var data = localStorage.getItem("data");
			if(data)
				port.postMessage({
					data:data
				});
		}else if (data.command == "FEED_INFO") {
			var feed = new FeedFinder().find();
//			console.log("feed:", feed, feed.length);
			if (feed.length > 2) {
//				console.log("post it?");
				port.postMessage({
					data : feed
				});
			}
		}
	});
});
