﻿var SNSFeed = function(info, callback) {
	this._mInfo = info;
	this._mCallback = callback;
	
	this._mPrivateFlags = 0;
	this._mTimer = null;
	this._mXHR = null;
};
SNSFeed.locallySerialize = function(s) {
};
SNSFeed.acceptable = function(authorities, url) {
	for (var i=0, len=authorities.length; i<len; i++) {
		if (url.indexOf(authorities[i]) != -1)
			return true;
	}
	return false;
};
SNSFeed.TIMEOUT = 2 * 60 * 1000;
SNSFeed.QUERY_SUCCESS = 0x00000001;
SNSFeed.QUERY_FAILURE = 0x00000002;
SNSFeed.QUERYING =      0x00000004;
SNSFeed.RESOLVED =      0x00000008;

SNSFeed.prototype = {
	updated : function() {
	},
	querying : function() {
		return (this._mPrivateFlags & SNSFeed.QUERYING) == SNSFeed.QUERYING;
	},
	_shutdownTimerService : function() {
		if (this._mTimer != null) {
			//TODO shutdown precede schedule
			clearTimeout(this._mTimer);
			this._mTimer = null;
			this._mXHR = null;
		}
	},
	scheduleQuery : function(url, callback, postData) {
		this._shutdownTimerService();
		this._mPrivateFlags |= SNSFeed.QUERYING;
		
		var xhr = new XMLHttpRequest(),
		_this = this,
		errorHandler = function(){
			callback.call(_this, SNSFeed.QUERY_FAILURE);
			//TODO clear flag
			_this._mPrivateFlags &= ~SNSFeed.QUERYING;
		};
		xhr.onerror = errorHandler;
		xhr.onabort = errorHandler;
		xhr.onreadystatechange = function() {
			if (xhr.readyState == 4) {
				_this._shutdownTimerService();
				if (xhr.status == 200) {
					callback.call(_this, SNSFeed.QUERY_SUCCESS, xhr.responseText);
					_this._mPrivateFlags &= ~SNSFeed.QUERYING;
				} else {
					errorHandler.call(_this);
				}
			}
		}
		this._mTimer = setTimeout(function(){
			//TODO the network connection is so terrible
			if (_this._mXHR)
				_this._mXHR.abort();
		}, SNSFeed.TIMEOUT);
		
		url += (Math.random().toFixed(5));
		if (postData != null) {
			xhr.open("POST", url, true);
			xhr.setRequestHeader("content-length", postData.length);
			xhr.setRequestHeader("content-type"," application/x-www-form-urlencoded");			
			xhr.send(postData);
		} else {
			xhr.open("GET", url, true);
			xhr.send(null);
		}
		_this._mXHR = xhr;
	},
	sendCallbackMessage : function(s) {
		if (this._mCallback)
			this._mCallback.call(this, s);
	}
};
var SinaSNSFeed = function(info, callback) {
	SNSFeed.call(this, info, callback);
	//TODO hold all tasks
	this._mPendingTasks = [SinaSNSFeed.FEED_URL];
};
SinaSNSFeed.APP_KEY = "1700178216";
SinaSNSFeed.AUTHORITIES = ["http://weibo.com", "http://t.sina.com", "http://t.sina.com.cn"];
SinaSNSFeed.POST_URL = "http://api.t.sina.com.cn/statuses/update.json?source=" + SinaSNSFeed.APP_KEY + "&rnd=";
SinaSNSFeed.FEED_URL = "http://api.t.sina.com.cn/statuses/public_timeline.json?source="
							+ SinaSNSFeed.APP_KEY +"&count=1&rnd="; 
SinaSNSFeed.prototype = {
	__proto__: SNSFeed.prototype,
	updated : function() {
		if (this._hasPendingTask())
			this._detect();
	},
	_hasPendingTask : function() {
		if ((this._mPrivateFlags & SNSFeed.RESOLVED) == SNSFeed.RESOLVED)
			return false;
		return this._mPendingTasks.length > 0;
	},
	_weiboUpdated : function(data) {
		try {
			var d = JSON.parse(data);
			var postAt = new Date(d[0].created_at).getTime();
			return postAt > this._mInfo[4];
		} catch (e) {
			return false;
		}
	},
	post : function(msg, callback) {
		var _this = this;
		
		var data = [];
		data.push("source", encodeURIComponent(SinaSNSFeed.APP_KEY));
		data.push("status", encodeURIComponent(msg));
		this.scheduleQuery(SinaSNSFeed.POST_URL, callback, data.join("&"));
	},
	_detect : function() {
		//TODO get a task
		var task = this._mPendingTasks.pop(),
		_this = this;
		
		this.scheduleQuery(task, function(status, result){
			if (status == SNSFeed.QUERY_FAILURE) {
				if (_this._hasPendingTask())
					_this._detect();
				else
					_this.sendCallbackMessage(status);
			} else {
				if (task == SinaSNSFeed.FEED_URL) {
					var s = _this._weiboUpdated(result);
//					console.log(result, "<<<<<<<<<<<", s);
					_this.sendCallbackMessage(s ? SNSFeed.QUERY_SUCCESS : SNSFeed.QUERY_FAILURE);
				} else {
					_this.sendCallbackMessage(status);
//					console.log("=========================================");
				}
			}
		});
	}
};
var SNSFeedManager = function() {
	this._mPrivateFlags = 0;
};
SNSFeedManager.instance = null;
SNSFeedManager.getInstance = function() {
	return SNSFeedManager.instance ? SNSFeedManager.instance
			: (SNSFeedManager.instance = new SNSFeedManager()); 
};
SNSFeedManager.POOL = [
	SinaSNSFeed             
];
SNSFeedManager.getFeed = function(info, callback) {
	var url = (info instanceof Array) ? info[3] : info;
	for (var i in SNSFeedManager.POOL) {
		var sns = SNSFeedManager.POOL[i];
		if (SNSFeed.acceptable(sns.AUTHORITIES, url)) {
			return new sns(info, callback);
		}
	}
	return null;
};
SNSFeedManager.prototype = {};