// error retry
// auto remove
// auto prepend
// permalink replace state

chrome.extension.sendMessage({ type: 'load', url: location.href }, function(res) {
    AutoPagerize.SITEINFO.concat(AutoPagerize.MICROFORMAT, res.val).some(init)
})

function AutoPagerize() {
}
AutoPagerize.SITEINFO = [
    /* sample
    {
        url:          'http://(.*).google.+/(search).+',
        nextLink:     'id("navbar")//td[last()]/a',
        pageElement:  '//div[@id="res"]/div',
        exampleUrl:   'http://www.google.com/search?q=nsIObserver',
    },
    */
    /* template
    {
        url:          '',
        nextLink:     '',
        pageElement:  '',
        exampleUrl:   '',
    },
    */
]
AutoPagerize.MICROFORMAT = {
    url:          '.*',
    nextLink:     '//a[@rel="next"] | //link[@rel="next"]',
    insertBefore: '//*[contains(@class, "autopagerize_insert_before")]',
    pageElement:  '//*[contains(@class, "autopagerize_page_element")]'
}

function init(siteinfo) {
    var nextLinkNode = getFirstElementByXPath(siteinfo.nextLink, document)
    if (!nextLinkNode || !nextLinkNode.href) {
        console.log('nextLink not found', siteinfo.nextLink)
        return false
    }
    var pageElement = getElementsByXPath(siteinfo.nextLink, document)
    if (pageElement.length === 0) {
        console.log('pageElement not found', siteinfo.pageElement)
        return false
    }
    return start(siteinfo, nextLinkNode.href, pageElement)
}

function start(siteinfo, nextLink, pageElement) {
    var insertPoint = findInsertPoint(siteinfo)
    var loadedURLs = {}
    loadedURLs[location.href] = true
    var isLoading = false
    var onscroll = throttle(function() {
        var wh = window.innerHeight
        var r = document.body.scrollHeight - window.scrollY - (wh * 2)
        if (r >= 0 || isLoading || loadedURLs[nextLink]) {
            return
        }
        isLoading = true
        getDocument(nextLink, function(err, doc) {
            loadedURLs[nextLink] = true
            if (!doc || !doc.URL) {
                return
            }
            if (!isSameOrigin(location, doc.URL)) {
                return
            }
            var n = getFirstElementByXPath(siteinfo.nextLink, doc)
            var nodes = getElementsByXPath(siteinfo.pageElement, doc)
            if (nodes.length === 0) {
                console.log('pageElement not found', siteinfo.pageElement)
                return
            }
            nodes.forEach(function(i) {
                insertPoint.parentNode.insertBefore(i, insertPoint)
            })
            nextLink = n && n.href
            isLoading = false
        })
    }, 1000)
    window.addEventListener('scroll', onscroll, false)
    onscroll()
    return true
}

function isSameOrigin(loc, url) {
    var a = document.createElement('a')
    a.setAttribute('href', url)
    return loc.protocol === a.protocol && loc.host === a.host
}

function findInsertPoint(siteinfo) {
    var r = null
    if (siteinfo.insertBefore) {
        r = getFirstElementByXPath(siteinfo.insertBefore)
    }
    if (!r) {
        var lastPageElement = getElementsByXPath(siteinfo.pageElement).pop()
        if (lastPageElement) {
            r = lastPageElement.nextSibling ||
                lastPageElement.parentNode.appendChild(document.createTextNode(' '))
        }
    }
    return r
}

function getDocument(url, callback) {
    var xhr = new XMLHttpRequest()
    xhr.onload = function (x) {
        callback(null, xhr.response)
    }
    xhr.error = function (err) {
        callback(err)
    }
    xhr.open('GET', url)
    xhr.responseType = 'document'
    xhr.send()
}

function throttle(f, interval) {
    var timer = null
    return function() {
        if (!timer) {
            timer = setTimeout(function() {
                f()
                timer = null
            }, interval || 500)
        }
    }
}

function getElementsByXPath(xpath, node) {
    var nodesSnapshot = getXPathResult(xpath, node,
        XPathResult.ORDERED_NODE_SNAPSHOT_TYPE || 7)
    var data = []
    for (var i = 0; i < nodesSnapshot.snapshotLength; i++) {
        data.push(nodesSnapshot.snapshotItem(i))
    }
    return data
}

function getFirstElementByXPath(xpath, node) {
    var result = getXPathResult(xpath, node,
        XPathResult.FIRST_ORDERED_NODE_TYPE || 9)
    return result.singleNodeValue
}

function getXPathResult(xpath, node, resultType) {
    node = node || document
    var doc = node.ownerDocument || node
    var resolver = doc.createNSResolver(node.documentElement || node)
    // Use |node.lookupNamespaceURI('')| for Opera 9.5
    var defaultNS = node.lookupNamespaceURI(null)

    if (defaultNS) {
        const defaultPrefix = '__default__'
        xpath = addDefaultPrefix(xpath, defaultPrefix)
        var defaultResolver = resolver
        resolver = function (prefix) {
            return (prefix == defaultPrefix) ?
                defaultNS : defaultResolver.lookupNamespaceURI(prefix)
        }
    }
    return doc.evaluate(xpath, node, resolver, resultType, null)
}

function addDefaultPrefix(xpath, prefix) {
    const tokenPattern = /([A-Za-z_\u00c0-\ufffd][\w\-.\u00b7-\ufffd]*|\*)\s*(::?|\()?|(".*?"|'.*?'|\d+(?:\.\d*)?|\.(?:\.|\d+)?|[\)\]])|(\/\/?|!=|[<>]=?|[\(\[|,=+-])|([@$])/g
    const TERM = 1, OPERATOR = 2, MODIFIER = 3
    var tokenType = OPERATOR
    prefix += ':'
    function replacer(token, identifier, suffix, term, operator, modifier) {
        if (suffix) {
            tokenType =
                (suffix == ':' || (suffix == '::' &&
                 (identifier == 'attribute' || identifier == 'namespace'))) ? MODIFIER : OPERATOR
        }
        else if (identifier) {
            if (tokenType == OPERATOR && identifier != '*') {
                token = prefix + token
            }
            tokenType = (tokenType == TERM) ? OPERATOR : TERM
        }
        else {
            tokenType = term ? TERM : operator ? OPERATOR : MODIFIER
        }
        return token
    }
    return xpath.replace(tokenPattern, replacer)
}
