//------------------------------------------------
// Message受信イベントリスナ
//------------------------------------------------
chrome.extension.onRequest.addListener(function (request, sender, sendResponse) {
    switch (request.command) {
        case "add": // BackgroundPageからReadingList追加通知
            // すでに表示されてたら消す
            $("#readmine_notify").remove();

            // 通知ダイアログ構築
            var dialog = $("<div />")
            .attr("id", "readmine_notify")
            .addClass("readmine_notify")
            .hide()
            // 外枠
            .append($("<div />")
                .attr("id", "readmine_notify_close")
                .css("background-image", "url(" + chrome.extension.getURL("images/close.png") + ")")
                .click(function () { dialog.fadeOut(200, function () { dialog.remove(); }); })
                .hide())
            // ロゴ
            .append($("<div />")
                .attr("id", "readmine_notify_logo")
                .css("background-image", "url(" + chrome.extension.getURL("images/icon_32.png") + ")")
            // ページタイトル
            .append($("<div />")
                .addClass("readmine_notify_message")
                .text(request.title))
            // タグ入力フォーム
            .append($("<div />")
                .attr("id", "readmine_notify_editor")
                .addClass("readmine_notify_editor")
                .html("<span>Edit tags here...</span>")
                .append($("<div />")
            // textbox
                    .append($("<input />")
                        .attr("id", "readmine_notify_taginput")
                        .attr("type", "text")
                        .keypress(function (e) {
                            if (e.keyCode == 13) {
                                $("#readmine_notify_edit").click();
                            }
                        }))
            // button
                    .append($("<input />")
                        .attr("id", "readmine_notify_edit")
                        .addClass("readmine_notify_edit")
                        .attr("type", "button")
                        .attr("value", "OK")
                        .click(function () {
                            $(this).attr("disabled", "disabled");
                            $("#readmine_notify_taginput").attr("disabled", "disabled");
                            var newTag = $("#readmine_notify_taginput").val();
                            newTag = newTag.replace(new RegExp(/["'{}[\]]/g), "");
                            if (newTag == "") { return false; }
                            var update_tags = "{\"0\":{\"url\":\"" + request.url + "\",\"tags\":\"" + newTag + "\"}}";
                            chrome.extension.sendRequest({ command: "update_tags", update_tags: update_tags }, function (response) {
                                $("#readmine_notify #readmine_notify_editor > span").text("Tags update " + response.message);
                                dialog.fadeOut(3000, function () { dialog.remove(); });
                            });
                        }))
                    .hide())));
            $("body").append(dialog);
            dialog.fadeIn(500);

            var t = setTimeout(function () {
                dialog.fadeOut(3000, function () { if (dialog.css("display") == "none") { dialog.remove(); } });
                clearTimeout(t);
            }, 3000);

            // ポップアップにホバーしたときにタグ入力フォームを表示させる
            dialog.one("mouseover", function () {
                clearTimeout(t);
                dialog.stop()
                .css("opacity", "1")
                .animate({ height: dialog.height() + 24 + "px" },
                    "300", function () {
                        $("#readmine_notify #readmine_notify_close").show();
                        $("#readmine_notify #readmine_notify_editor > div").show();
                        $("#readmine_notify #readmine_notify_editor > span").text("Tags (Separate with comma)");
                        $("#readmine_notify #readmine_notify_taginput").focus();
                    });


            });
            break;

        case "error": // エラー通知追加したい[ToDo]
            break;
    }


    // 一応空でも返したほうがいいらしい
    sendResponse({});
});

//------------------------------------------------
// ショートカットキー
//------------------------------------------------
//$(document).keyup(function (e) { 
//    if (!e.altKey && e.shiftKey && e.ctrlKey && e.keyCode == 76) {
//        alert("Press Ctrl+Shift+L");
//    }
//});

//------------------------------------------------
// ページ読み込み完了時イベント
//------------------------------------------------
$(document).ready(function (e) {
    chrome.extension.sendRequest({ command: "page_init" }, function (response) {
        // GoogleReader統合
        if (response.integrateGoogleReader == "true" && isGoogleReaderURL()) {
            $("#entries")
            .bind("DOMNodeInserted", function (ev) { handleDOMNodeInserted(ev); })
            .find(".entry")
            .each(function (index, value) {
                insertReadmineAction($(value));
            });
        }

        // コントロールパネル表示
        // 実装予定
        //if (response.hasUrl) {
        //alert(response.url);
        //}
    });

    return false;
});

//------------------------------------------------
// GoogleReaderのURLかチェック
//------------------------------------------------
function isGoogleReaderURL() {
    // GoogleReaderのURLをチェック
    var url = location.href;
    var isGoogleReader1 = /^http(:?s?):\/\/www\.google\.[^\/]+\/reader\//; // http(s)://www.google.xxx/reader/
    var isGoogleReader2 = /^http(:?s?):\/\/reader\.google\./; // http(s)://reader.google.xxx
    if (url.match(isGoogleReader1) || url.match(isGoogleReader2)) {
        return true;
    }

    return false;
}

//------------------------------------------------
// DOMNodeInsertedイベントハンドラ
//------------------------------------------------
function handleDOMNodeInserted(e) {
    var target = $(e.target);
    if (target.hasClass("entry")) {
        insertReadmineAction(target);
    }
}

//------------------------------------------------
// GoogleReaderにReadmineアイコンを追加
//------------------------------------------------
function insertReadmineAction(entry) {
    var entry_link = $(".entry-title-link", entry);
    var entry_url = entry_link.attr("href");
    var entry_title = entry_link.text();

    if (entry_url) {
        $(".entry-actions", entry)
        .append($("<span />")
            .text("Readmine")
            .addClass("gr_add")
            .addClass("link")
            .css("background-image", "url(\"" + chrome.extension.getURL("images/markasread.png") + "\")")
            .data("url", entry_url)
            .data("title", entry_title)
            .click(function (e) {
                chrome.extension.sendRequest({ command: "add_page", url: entry_url, title: entry_title }, function (response) {
                    $(e.target).addClass("gr_add_checked");
                });
            }));
    } else {
        setTimeout(function () { insertReadmineAction(entry); }, 100);
    }
}


