/**
 * @author: Mitsuaki Ishimoto
 * @see: http://blog.justoneplanet.info/
/*=================================================================*/
(function(chrome){
    var parent;
    var kaomoji = {};
    kaomoji.favoriteDB = null;
    kaomoji.historyDB  = null;
    kaomoji.dbVersion = 3;
    kaomoji.dbError = function (e) {
        console.log(e);
    }
    kaomoji.createContextMenu = function(){
        var parent = chrome.contextMenus.create(
            {
                "title"    : "\u30DE\u30A4\u9854\u6587\u5B57",
                "contexts" : ["editable"]
            },
            function(){
                var ary = [];
                var create = function (ary) {
                    ary.forEach(function(e, i, ary){
                        chrome.contextMenus.create({
                            "title"    : e,
                            "parentId" : parent,
                            "contexts" : ["editable"],
                            "onclick"  : function(info, tab){
                                selected = e;
                                chrome.tabs.executeScript(
                                    null,
                                    {
                                        "file"      : "content.js",
                                        "allFrames" : true
                                    },
                                    function(){// callback
                                        //log(i);
                                    }
                                );
                            }
                        });
                    });
                }
                kaomoji.getAllFavorite(function (e) {
                    var result = e.target.result;
                    if (!!result == false) {
                        create(ary);
                        return;
                    }
                    else {
                        ary.push(result.value.face);
                        result.continue();
                    }
                });
            }
        );
        return parent;
    }
    kaomoji.openFavorite = function () {
        var request = webkitIndexedDB.open("favorite", kaomoji.dbVersion);
        request.onupgradeneeded = function (e) {
            kaomoji.favoriteDB = e.target.result;
            var db = kaomoji.favoriteDB;
            db.createObjectStore("favorite", {"keyPath" : "timeStamp"});
        }
        request.onsuccess = function (e) {
            kaomoji.favoriteDB = e.target.result;
            var db = kaomoji.favoriteDB;
            kaomoji.getAllFavorite();
            parent = kaomoji.createContextMenu();
        }
        request.onfailure = kaomoji.dbError;
    };
    kaomoji.openHistory = function () {
        var request = webkitIndexedDB.open("history");
        console.log(request);
        request.onupgradeneeded = function (e) {
            kaomoji.favoriteDB = e.target.result;
            var db = kaomoji.favoriteDB;
            db.createObjectStore("history", {"keyPath" : "timeStamp"});
        }
        request.onsuccess = function (e) {
            kaomoji.historyDB = e.target.result;
            var db = kaomoji.historyDB;
            kaomoji.getAllHistory();
        }
        request.onfailure = kaomoji.dbError;
    };
    kaomoji.addFavorite = function (timeStamp, face, tag) {
        var db    = kaomoji.favoriteDB;
        var trans = db.transaction(["favorite"], "readwrite");
        var store = trans.objectStore("favorite");
        var request = store.put({
            "face"      : face,
            "tag"       : tag,
            "timeStamp" : timeStamp
        });
        request.onsuccess = function (e) {
            console.log(e);
        }
        request.onerror = function (e) {
            console.log(e);
        }
    }
    kaomoji.addHistory = function (face, tag) {
        var db    = kaomoji.historyDB;
        var trans = db.transaction(["history"], "readwrite");
        var store = trans.objectStore("history");
        var request = store.put({
            "face"      : face,
            "tag"       : tag,
            "timeStamp" : Date.now()
        });
        request.onsuccess = function (e) {
            console.log(e);
        }
        request.onerror = function (e) {
            console.log(e);
        }
    }
    kaomoji.getAllHistory = function (callback) {
        var db = kaomoji.historyDB;
        var trans = db.transaction(["history"], "readonly");
        var store = trans.objectStore("history");
        kaomoji.deleteHistory();
        
        var keyRange = webkitIDBKeyRange.lowerBound(0);
        var cursorRequest = store.openCursor(keyRange);
        cursorRequest.onsuccess = callback;
        cursorRequest.error = kaomoji.dbError;
    }
    kaomoji.getAllFavorite = function (callback) {
        var counter = 0;
        var length = localStorage.length;
        for(var i in localStorage){
            try{
                if(i){
                    kaomoji.addFavorite(counter, i, JSON.parse(localStorage[i]));
                    counter++;
                }
            }
            catch(e){
            }
        }
        if (counter > 0 && counter === length) localStorage.clear();
        var db = kaomoji.favoriteDB;
        var trans = db.transaction(["favorite"], "readonly");
        var store = trans.objectStore("favorite");
        
        var keyRange = webkitIDBKeyRange.lowerBound(0);
        var cursorRequest = store.openCursor(keyRange);
        cursorRequest.onsuccess = callback;
        cursorRequest.error = kaomoji.dbError;
    }
    kaomoji.deleteFavorite = function (key, callback) {
        var db = kaomoji.favoriteDB;
        var trans = db.transaction(["favorite"], "readwrite");
        var store = trans.objectStore("favorite");
        var request = store.delete(parseInt(key, 10));
        request.onsuccess = function (e) {
            callback(key);
        }
        request.onerror = kaomoji.dbError;
    }
    kaomoji.deleteHistory = function () {
        var db = kaomoji.historyDB;
        var trans = db.transaction(["history"], "readwrite");
        var store = trans.objectStore("history");
        
        var keyRange = webkitIDBKeyRange.upperBound(Date.now() - 1000 * 60 * 60 * 24 * 30);
        var cursorRequest = store.openCursor(keyRange);
        cursorRequest.onsuccess = function (e) {
            var result = e.target.result;
            if (!!result == false) {
                return;
            }
            store.delete(result.value.timeStamp);
            result.continue();
        }
        cursorRequest.error = kaomoji.dbError;
    }
    kaomoji.openFavorite();
    kaomoji.openHistory();
    
    
    var selected;
    /**
     * log
     * send the data of using a kaomoji
     * @param string kaomoji
     */
    var log = function(kaomoji){
        var xhr = new XMLHttpRequest();
        xhr.open("POST", "http://kaomoji.justoneplanet.info/logs/add", true);
        xhr.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
        xhr.onreadystatechange = function() {
            if(xhr.readyState === 4){
            }
        }
        xhr.send("kaomoji=" + encodeURIComponent(kaomoji));            
    }
    
    
    chrome.extension.onRequest.addListener(
        function(request, sender, sendResponse){
            switch(request.action){
                case 'clearKaomoji':
                    selected = null;
                    sendResponse(true);
                    break;
                case 'getKaomoji':
                    sendResponse(selected);
                    break;
                case 'getParent':
                    sendResponse(parent);
                    break;
                case 'resetMenu':
                    chrome.contextMenus.removeAll(function(){
                        parent = kaomoji.createContextMenu();
                    });
                    break;
                case 'log':
                    var k = request.kaomoji;
                    log(k);
                    break;
                case 'kaomoji.addFavorite':
                    kaomoji.addFavorite(request.timeStamp, request.face, request.tag);
                    break;
                case 'kaomoji.addHistory':
                    kaomoji.addHistory(request.face, request.tag);
                    break;
                case 'kaomoji.deleteFavorite':
                    kaomoji.deleteFavorite(request.key, sendResponse);
                    break;
                case 'kaomoji.deleteHistory':
                    kaomoji.deleteHistory();
                    break;
                case 'kaomoji.getAllFavorite':
                    var ary = [];
                    kaomoji.getAllFavorite(function (e) {
                        var result = e.target.result;
                        if (!!result == false) {
                            sendResponse(ary);
                            return;
                        }
                        else {
                            ary.push(result.value);
                            result.continue();
                        }
                    });
                    break;
                case 'kaomoji.getAllHistory':
                    var ary = [];
                    kaomoji.getAllHistory(function (e) {
                        var result = e.target.result;
                        if (!!result == false) {
                            sendResponse(ary);
                            return;
                        }
                        else {
                            ary.push(result.value);
                            result.continue();
                        }
                    });
                    break;
            }
        }
    );
})(chrome);
